@@ -4,13 +4,18 @@ proc helpMsg {field} {
 set help_text(TkGnats_Version) "\n$TkGnats(tkgnats_version)\n\nYou are running [info nameofexecutable] patchlevel [info patchlevel]."
 
 if {$field == "TkGnats_Version"} {
+    if {[info exists TkGnats(Servers)]} {
+        append help_text($field) "\n\nServers: $TkGnats(Servers)"
+    }
     show_help $field $help_text($field)
     return
 }
 
 set help_text(TkGnats_Variables) $help_text(TkGnats_Version)\n
 foreach t [lsort [array names TkGnats]] {
-    append help_text(TkGnats_Variables) "[format "\n%-24s:" $t] $TkGnats($t)"
+    if {[string first "Password" $t] < 0} {
+        append help_text(TkGnats_Variables) "[format "\n%-24s:" $t] $TkGnats($t)"
+    }
 }
 
 set help_text(TkGnats_About) $help_text(TkGnats_Version)
@@ -75,6 +80,7 @@ append help_text(TkGnats_About) "\nTKGNATSINI:          $ini"
 append help_text(TkGnats_About) "\nWISHPATH:            $TkGnats(WISHPATH)"
 append help_text(TkGnats_About) "\nTKGNATSLIB:          $TkGnats(lib)"
 append help_text(TkGnats_About) "\nSiteServerDir:       $TkGnats(SiteServerDir)"
+append help_text(TkGnats_About) "\nUserAccess:          $TkGnats(UserAccess)"
 append help_text(TkGnats_About) "\nUserDir:             $TkGnats(UserDir)"
 append help_text(TkGnats_About) "\nUserServerDir:       $TkGnats(UserServerDir)"
 append help_text(TkGnats_About) "\nedit_authorized:     $TkGnats(edit_authorized)"
@@ -136,7 +142,7 @@ Entry Fields: Originator, Reply-To, Release and Synopsis
 
 set help_text(Edit_Text_Fields) {
 Text Fields: Description, How-To-Repeat, Environment, Audit-Trail,
-             Unformatted and Fix
+             Unformatted, Fix and Release-Note
 
      These are the multi-line, free-format text entry fields of a
      problem report.
@@ -157,7 +163,7 @@ Overview of Viewing Problem Reports
   
 set help_text(View_Text_Fields) {
 Text Fields: Description, How-To-Repeat, Environment, Audit-Trail,
-             Unformatted and Fix
+             Unformatted, Fix and Release-Note
 
      These are the multi-line, free-format text entry fields of a
      problem report.
@@ -167,6 +173,7 @@ Text Fields: Description, How-To-Repeat, Environment, Audit-Trail,
      to get help for that field.
 }
 
+#TTD: Yikes! How can this (change-request) be dynamic?
 set help_text(Create_Overview) {
 Overview of Creating Problem Reports
 
@@ -217,7 +224,8 @@ Entry Fields: Originator, Reply-To, Release and Synopsis
      progress of this bug report. They will be copied on all
      email. The list must be comma-separated.
 }
-   
+
+#TTD: Yikes! How can this (change-request) be dynamic?
 set help_text(Create_Text_Fields) {
 Text Fields: Description, How-To-Repeat, Environment and Fix
 
@@ -253,6 +261,12 @@ Overview of the Query System
      Leaving any field blank, or not selecting any check buttons (Class,
      State, etc), means that field is unrestricted in the search.
 
+     Be warned that searching the "Text-Fields" causes the entire bug
+     report to be read (by the server) and will take much longer to
+     run.  Be sure to restrict the search by specifying other search
+     criteria, and thereby minimizing the number of reports that have
+     to actually be read.
+
      Pressing the "Return" key in the entry fields is a shortcut
      for the "Do Query" button.
 }
@@ -333,8 +347,14 @@ Entry Widgets: Synopsis, Release, Originator and Text-Fields
 
      The Text-Fields are the multi-line, free-format text entry fields
      of a problem report such as Description, How-To-Repeat,
-     Environment, Audit-Trail, Unformatted and Fix. They are all searched
-     with the one regular expression that you enter.
+     Environment, Audit-Trail, Unformatted, Fix and Release-Note. They
+     are all searched with the one regular expression that you enter.
+
+     Be warned that searching the "Text-Fields" causes the entire bug
+     report to be read (by the server) and will take much longer to
+     run.  Be sure to restrict the search by specifying other search
+     criteria, and thereby minimizing the number of reports that have
+     to actually be read.
 
      Note that some fields, such as Originator, don't show their entire
      contents in the Query Listbox for space reasons. Thus, you may get
@@ -453,7 +473,9 @@ Menu Bar
      Actions
 
           These actions apply _only_ to the problem report currently
-          selected (highlighted) in the Query Listbox.
+          selected (highlighted) in the Query Listbox. You can get
+          this same menu to pop-up by clicking the right mouse button
+          on any report in the listbox.
 
           Edit...          - invoke the problem report editor.
           View...          - invoke the problem report viewer.
@@ -481,7 +503,10 @@ Query Listbox
      set according to the fields that you select in the "Fields"
      menu on the menubar.
 
-     Click the left mouse button on any problem report to "select" it.
+     Click the left  mouse button on any problem report to "select" it.
+
+     Click the right mouse button on any problem report to pop-up the
+     Actions menu.
 
      Actions, such as editing or sending follow-up email, can be found
      under the "Actions" menubar item. The actions there are performed
@@ -506,8 +531,14 @@ Text-Fields
 
      These are the multi-line, free-format text entry fields of a
      problem report such as Description, How-To-Repeat, Environment,
-     Audit-Trail, Unformatted and Fix.
+     Audit-Trail, Unformatted, Fix and Release-Note.
     
+     Be warned that searching the "Text-Fields" causes the entire bug
+     report to be read (by the server) and will take much longer to
+     run.  Be sure to restrict the search by specifying other search
+     criteria, and thereby minimizing the number of reports that have
+     to actually be read.
+
      In Create, View and Edit modes, the MultiText fields are shown
      one at a time.  Click on the field buttons to change fields. When
      the button is depressed, click again to get help for that field.
@@ -516,33 +547,12 @@ Text-Fields
      selector.  Select a file to insert at the cursor position.
 }
 
-set help_text(State) {
-"open"
-     The initial state of a Problem Report.  This means the PR has been
-     filed and the responsible person(s) notified.
-
-"analyzed"
-     The responsible person has analyzed the problem.  The analysis
-     should contain a preliminary evaluation of the problem and an
-     estimate of the amount of time and resources necessary to solve
-     the problem.  It should also suggest possible workarounds.
-
-"feedback"
-     The problem has been solved, and the originator has been given a
-     patch or other fix.  The PR remains in this state until the
-     originator acknowledges that the solution works.
-
-"closed"
-     A Problem Report is closed ("the bug stops here") only when any
-     changes have been integrated, documented, and tested, and the
-     submitter has confirmed the solution.
-
-"suspended"
-     Work on the problem has been postponed.  This happens if a timely
-     solution is not possible or is not cost-effective at the present
-     time.  The PR continues to exist, though a solution is not being
-     actively sought.  If the problem cannot be solved at all, it
-     should be closed rather than suspended.
+set help_text(State) "\n`>State:'\n     (ENUMERATED) The current state of the PR.  Accepted values are:\n"
+foreach state $TkGnats(StatesFile) {
+    append help_text(State) "\n    `[lindex [split $state ":"] 0]'\n          [lindex [split $state ":"] 2]\n"
+    if {[lindex [split $state ":"] 1] != ""} {
+        append help_text(State) "\n          This state has the state type \"[lindex [split $state ":"] 1]\".\n"
+    }
 }
 
 set help_text(Submitter-Id) {
@@ -572,7 +582,7 @@ set help_text(Reply-To) {
 set help_text(Organization) {
 `>Organization:'
      (MULTITEXT) The originator's organization.  The default value is
-     set with the variable `DEFAULT_ORGANIZATION' in the `config' file.
+     set with the variable `TkGnats(ORGANIZATION)' in the `config' file.
 }
 
 set help_text(Confidential) {
@@ -645,31 +655,12 @@ set help_text(Category) {
      Site.
 }
 
-set help_text(Class) {
-`>Class:'
-     (ENUMERATED) The class of a problem can be one of the following:
-
-    `sw-bug'
-          A general product problem.  (`sw' stands for "software".)
-
-    `doc-bug'
-          A problem with the documentation.
-
-    `change-request'
-          A request for a change in behavior, etc.
-
-    `support'
-          A support problem or question.
-
-    `duplicate (PR-NUMBER)'
-          Duplicate PR.  PR-NUMBER should be the number of the original
-          PR.
-
-    `mistaken'
-          No problem, user error or misunderstanding.  This value is
-          valid only at the Support Site.
-
-     The default is `sw-bug'.
+set help_text(Class) "\n`>Class:'\n     (ENUMERATED) The current class of the PR.  Accepted values are:\n"
+foreach class $TkGnats(ClassesFile) {
+    append help_text(Class) "\n    `[lindex [split $class ":"] 0]'\n          [lindex [split $class ":"] 2]\n"
+    if {[lindex [split $class ":"] 1] != ""} {
+        append help_text(Class) "\n          This class has the class type \"[lindex [split $class ":"] 1]\".\n"
+    }
 }
 
 set help_text(Release) {
@@ -685,6 +676,20 @@ set help_text(Keywords) {
      here can be queried directly.
 }
 
+set help_text($TkGnats(Quarter)) "
+`>$TkGnats(Quarter):'
+     (TEXT) This field is available if your GNATS system has been
+     configured with the flag \"--with-released-based\". Text entered
+     here can be queried directly.
+"
+
+set help_text(Date-Required) {
+`>Date-Required:'
+     (TEXT) This field is available if your GNATS system has been
+     configured with the flag "--with-released-based". Dates entered
+     here can be queried directly.
+}
+
 set help_text(Environment) {
 `>Environment:'
      (MULTITEXT) Description of the environment where the problem
@@ -710,6 +715,13 @@ set help_text(How-To-Repeat) {
      information can point the way toward a solution.
 }
 
+set help_text(Release-Note) {
+`>Release-Note:'
+     (MULTITEXT) Human readable (user friendly) comments about
+     problems or fixes suitable for incorporation into release notes
+     for a given software distribution.
+}
+
 set help_text(Fix) {
 `>Fix:'
      (MULTITEXT) A description of a solution to the problem, or a patch
@@ -734,30 +746,6 @@ set help_text(Number) {
      are named by category.
 }
 
-set help_text(XXXState) {
-`>State:'
-     (ENUMERATED) The current state of the PR.  Accepted values are:
-
-    `open'
-          The PR has been filed and the responsible person notified.
-
-    `analyzed'
-          The responsible person has analyzed the problem.
-
-    `feedback'
-          The problem has been solved, and the originator has been
-          given a patch or other fix.
-
-    `closed'
-          The changes have been integrated, documented, and tested, and
-          the originator has confirmed that the solution works.
-
-    `suspended'
-          Work on the problem has been postponed.
-
-     The initial state of a PR is `open'.
-}
-
 set help_text(Responsible) {
 `>Responsible:'
      (TEXT) The person responsible for this category.  GNATS retrieves
@@ -775,6 +763,11 @@ set help_text(Last-Modified) {
      (TEXT) The time that this PR was last edited or changed.
 }
 
+set help_text(Closed-Date) {
+`>Closed-Date:'
+     (TEXT) The time that this PR was closed.
+}
+
 set help_text(Audit-Trail) {
 `>Audit-Trail:'
      (MULTITEXT) Tracks related electronic mail as well as changes in
