@@ -29,6 +29,7 @@ static void reply_to_submitter	PARAMS((char *));
 static void create_report	PARAMS((char *, int));
 static int get_bug_number	PARAMS((void));
 static void notify_responsible	PARAMS((char*, char*, char*, char*, char*, int, struct tm *));
+static char* derive_submitter  PARAMS((void));
 
 static struct bad_enum *bad_enums = NULL;
 
@@ -49,6 +50,8 @@ gnats (fp)
   char *site, *bug_group;
   char *subject = NULL, *synopsis = NULL;
   char *p;
+  char message[STR_MAX];
+  char *from_address;
 
   mode_t mode;
   struct stat sbuf;
@@ -68,10 +71,20 @@ gnats (fp)
   read_pr (fp, 0);
   fclose (fp);
 
+/*  If we don't have a valid submitter ID, try to get it from the "From:"
+    header.  If that doesn't work, use the default from the config file.  */
+
   site = field_value (SUBMITTER);
+  if ((find_submitter (&submitter, site) == -1) 
+      || (strcmp (site, def_subm) == 0))
+  {
+    site = derive_submitter (); 
+    set_field (SUBMITTER, site);
+  }
 
   if (find_submitter (&submitter, site) == -1)
     {
+
       log_msg (LOG_INFO, 1, "resetting to default submitter from:", site);
       site = def_subm;
       set_field (SUBMITTER, def_subm);
@@ -84,15 +97,16 @@ gnats (fp)
   bug_group = field_value (CATEGORY);
   if (find_category (&category, bug_group) == -1)
     {
-      log_msg (LOG_INFO, 1, "resetting bug category to pending from:",
-	       bug_group);
-      bug_group = PENDING;
-      set_field (CATEGORY, PENDING);
+      sprintf (message, "%s from: %s\n", default_category, bug_group);
+      log_msg (LOG_INFO, 1, "resetting bug category to ", message);
+      bug_group = default_category;
+      set_field (CATEGORY, default_category);
 
-      /* Again, but with `pending' now.  If it's not there, kick this to
+      /* Again, but with default now.  If it's not there, kick this to
 	 the GNATS_ADMIN.  */
       if (find_category (&category, bug_group) == -1)
-	punt (1, "No `pending' directory under gnats-root (%s).", gnats_root);
+	punt (1, "No `%s' directory under gnats-root (%s).", 
+	      default_category, gnats_root);
     }
 
   /* Set responsible field, adding full name as well (if found).  If it's
@@ -174,7 +188,7 @@ gnats (fp)
 
   /* Set arrival date and time of response.  */
   seconds = time ((time_t) 0);
-  strftime (arrival_time, GNATS_TIME_LENGTH, "%a %b %e %T %Z %Y",
+  strftime (arrival_time, GNATS_TIME_LENGTH, "%a %b %d %H:%M:%S %Z %Y",
 	    localtime (&seconds));
   set_field (ARRIVAL_DATE, arrival_time);
 
@@ -184,34 +198,35 @@ gnats (fp)
     expired = get_response_time (submitter.rtime);
 
   /* Put together the path to where the bug will be stored.  If the dir
-     is not there, and the category is PENDING, auto-create that one,
+     is not there, and the category is the default, auto-create that one,
      if we want to.  If not, make the bug pending, and store in there.  */
   sprintf (path, "%s/%s", gnats_root, bug_group);
   err = stat (path, &sbuf);
   if (err == -1 && !flag_autocreate)
     {
-      bug_group = PENDING;
-      set_field (CATEGORY, PENDING);
-      log_msg (LOG_INFO, 1, "directory does not exist, changing to pending:",
+      bug_group = default_category;
+      set_field (CATEGORY, default_category);
+      log_msg (LOG_INFO, 1, "directory does not exist, changing to default:",
 	       path);
       sprintf (path, "%s/%s", gnats_root, bug_group);
       err = stat (path, &sbuf);
     }
 
-  /* Check ERR again, to see if pending was there.  */
+  /* Check ERR again, to see if default category was there.  */
   if (err == -1)
     {
-      if (strcmp (bug_group, PENDING) == 0)
+      if (strcmp (bug_group, default_category) == 0)
 	{
-	  log_msg (LOG_INFO, 0, "pending does not exist, creating...");
+	  log_msg (LOG_INFO, 1, default_category, 
+		   " does not exist, creating...");
 #ifdef S_IRWXU
 	  mode = S_IRWXU | S_IRGRP | S_IXGRP;
 #else
 	  mode = 0750;
 #endif
 	  if (mkdir (path, mode) == -1)
-	    punt (1, "Can't create `pending' directory under gnats_root (%s).",
-		  gnats_root);
+	    punt (1, "Can't create `%s' directory under gnats_root (%s).", 
+		  default_category, gnats_root);
 	}
       else if (flag_autocreate)
 	{
@@ -244,7 +259,7 @@ gnats (fp)
   log_msg (LOG_INFO, 1, "PR written out:", path);
 
   /* Add a line into the index for use by query-pr.  */
-  /* This should be done regardless of whether the bug is pending or not.  */
+  /* This should be done regardless of whether the bug is default or not.  */
   add_to_index ();
 
   /* If it fell into pending, don't send all of the mail for it, don't create
@@ -374,10 +389,11 @@ check_if_reply ()
   char token[STR_MAX];
   char *path;
   struct stat buf;
-  char *s, *c;
+  Category cat;
+  char *s, *c, *cat_name;
   struct re_pattern_buffer regex;
   struct re_registers regs;
-  int i, start, end;
+  int i, start, end, res;
   char *case_fold;
 
   s = header_value (SUBJECT);
@@ -396,11 +412,11 @@ check_if_reply ()
     case_fold[i] = tolower(i);
   regex.translate = case_fold;
 
-  /* too restrictive? */
-#define PAT "(.*re[ \t]*(\\[[0-9]+\\])?:)?[ \t]*([-a-z0-9_+.]*/([0-9]+))"
+  /* look for "category/1234" and "category: 1234" */
+#define PAT "(.*re[ \t]*(\\[[0-9]+\\])?:)?[ \t]*([-a-z0-9_+.]*[:/][ \t]*([0-9]+))"
+
   re_compile_pattern (PAT, strlen (PAT), &regex);
   i = re_match (&regex, s, strlen (s), 0, &regs);
-
   regfree (&regex);
 
   if (i <= 0)
@@ -415,8 +431,50 @@ check_if_reply ()
   s[end] = '\0';
 
   c = strchr (s + start, '/');
+  if (!c)
+    c = strchr (s + start, ':');
+
   if (c)
-    path = get_category (c + 1);
+    {
+      char sep = *c;
+
+      /* The category's name is what comes before the '/'. */
+      cat_name  = s + start;
+
+      /* Terminate it temporarily: */
+      *c = '\0';
+
+      /* Void cat just in case free_category() expects NULLs. */
+      memset ((void *) &cat, 0, sizeof (Category));
+
+      /* See if the category exists: */
+      res = find_category (&cat, cat_name);
+
+      /* Folks often send in "pr/1234" instead of a valid category */
+      if (res < 0 && !strcasecmp(cat_name, "pr"))
+	  res = 0;			/* pretend we found it in list */
+
+      /* Undo temporary terminator: */
+      *c = sep;
+
+      /* If no such category, then this is not a reply to a valid
+       * problem report.  This situtation can arise, for example, when
+       * someone has the string "OS/2" in their Subject header.
+       */
+      if (res == -1)
+        return NULL;
+      else
+        {
+          /* We only needed res, never cat, so free cat. */
+          free_category (&cat);
+
+	  c++;				/* skip sep */
+	  while (isspace(*c))		/* skip any whitespace */
+	      c++;
+
+          path = lookup_pr_path (c);
+        }
+    }
   else
     {
       path = xmalloc (PATH_MAX);
@@ -430,7 +488,87 @@ check_if_reply ()
 
   return path;
 }
-
+
+/*  This function compares the e-mail address in the "From:" header of a new
+    PR with entries in the "addresses" file.  If there is a match, we return
+    the submitter ID from "addresses", otherwise we return the default
+    submitter.   */
+
+static char *
+derive_submitter ()
+{
+  char *from_address;
+  char from_string[STR_MAX];
+  char *token = (char *) xmalloc (STR_MAX);
+  char path[STR_MAX];
+  char *compare;
+
+  char *name, *alias, *s, *t;
+  int i, start, end;
+
+  FILE *fp;
+
+  from_address = header_value (FROM);
+  strcpy (from_string, from_address);
+
+  if ((*from_string == '\0') || (*from_string == '\n'))
+  {
+    xfree (token);
+    return def_subm;
+  }
+  
+  s = from_string + strlen(from_string) - 2;
+  if (*s == '>')
+  {
+    *s = '\0';
+    for (t = s; *t != '<'; t--)
+      *t = toupper (*t);
+  }
+  else
+    for (t = from_string; *t != '\0'; t++)
+    {
+      *t = toupper (*t);
+      if ((*t == ' ') || (*t == '\n'))
+	*t = '\0';
+    }
+  sprintf (path, "%s/gnats-adm/addresses", gnats_root);
+  if ((fp = fopen (path, "r")) == (FILE *) NULL)
+  {
+    xfree (token);
+    return def_subm;
+  }
+  while ((fgets (token, STR_MAX, fp)) != NULL)
+    if (*token != '#')
+    {
+      name = token;
+      
+      for (alias = token; (*alias != ':') && (*alias != '\0'); alias++);
+      if (*alias == ':')
+      {
+	*alias = '\0';
+	if (*(++alias) != '\0')
+	  for (t = alias; *t != '\0'; t++)
+	  {
+	    *t = toupper (*t);
+	    if (*t == '\n')
+	      *t = '\0';
+	  }
+      }
+
+      compare = from_string + strlen (from_string) - strlen (alias);
+      if (strcmp (compare, alias) == 0)
+      {
+	fclose (fp);
+	return name;
+      }
+    }
+
+  xfree (token);
+  fclose (fp);
+  return def_subm;
+}
+
+
 /* Send a reply to the submitter of the bug.  */
 
 static void
@@ -450,13 +588,13 @@ reply_to_submitter (responsible)
   if (*reply_to == '\0')
     reply_to = header_value (FROM);
   fprintf (msg, "To: %s", reply_to);
-  fprintf (msg, "From: %s\n", gnats_addr);
+  fprintf (msg, "From: %s\n", gnats_admin);
   subject = header_value (SUBJECT);
   fprintf (msg, "Subject: Re: %s/%s: %s",
 	   field_value (CATEGORY), field_value (NUMBER), subject);
   if (*subject == '\0')
     fprintf (msg, "\n");
-  fprintf (msg, "Reply-To: %s, %s\n", gnats_addr, responsible);
+  fprintf (msg, "Reply-To: %s, %s\n", gnats_admin, responsible);
   fprintf (msg, "In-Reply-To: Your message of %s\t%s\n",
 	   header_value (DATE), header_value (MSG_ID));
 
@@ -606,7 +744,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
 	     header_value (DATE), header_value (MSG_ID));
 
 #else /* ! DONT_USE_RESENT */
-  fprintf(msg, "Resent-From: %s (GNATS Management)\n", gnats_user);
+  fprintf(msg, "Resent-From: %s (GNATS Management)\n", gnats_admin);
   fprintf(msg, "Resent-To: %s\n", responsible);
   if (*notify)
     fprintf(msg, "Resent-Cc: %s\n", notify);
@@ -614,7 +752,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
     char * reply_to = header_value (REPLY_TO);
     if (! (reply_to && *reply_to))
       reply_to = header_value (FROM);
-    fprintf(msg, "Resent-Reply-To: %s, %s", gnats_addr, reply_to);
+    fprintf(msg, "Resent-Reply-To: %s, %s", gnats_admin, reply_to);
   }
   for (i = RETURN_PATH; i < NUM_HEADER_ITEMS; i++)
     {
@@ -690,6 +828,22 @@ The filename for it is %s.", filename);
   unblock_signals ();
 }
 
+
+/* Get a pr number from its number (just uses basename).
+   Result is allocated, caller may free. */
+static char *
+pr_number_from_path (path)
+     char *path;
+{
+  char *result;
+  result = (char *) basename (path);
+
+  if (result != NULL)
+    return strdup (result);
+  else
+    return NULL;
+}
+
 /* Append a reply to an existing report.  The entire body of the message
    is appended, while just the To:, From:, Date:, and Subject: lines from
    the header are included.  We used to send notification of the reply
@@ -703,14 +857,18 @@ append_report (infile, filename)
      char *filename;
 {
   FILE *pr_file, *msg;
-  char *line = (char *) xmalloc (STR_MAX);
+  char *line = (char *) xmalloc (STR_MAXLONG);
   char *buf, *b;
   int buf_len, buf_max, l;
   Responsible *responsible;
   Submitter submitter;
   char *p, *q;
-  char *from, *to, *cc = " ", *subject = " ", *date;
+  char *number;
+  char *from, *to, *cc = " \n", *subject = " \n", *date;
   time_t t;
+  /* Where to keep the static index if necessary.  */
+  Index *current_index = (Index *)NULL;
+  Index *i, *prev_index = NULL, *old_index = NULL;
 
   /* Save the values we read while in main().  */
   from = (char *) strdup (header_value (FROM));
@@ -726,7 +884,7 @@ append_report (infile, filename)
     {
       t = time((time_t) 0);
       date = (char *) xmalloc (GNATS_TIME_LENGTH);
-      strftime (date, GNATS_TIME_LENGTH, "%a, %d %b %y %T %Z\n", localtime(&t));
+      strftime (date, GNATS_TIME_LENGTH, "%a, %d %b %y %H:%M:%S %Z\n", localtime(&t));
     }
 
   /* Now read in the PR itself.  */
@@ -759,7 +917,7 @@ append_report (infile, filename)
   APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
   APPEND_STRING ((char*)"\n", buf, b, buf_len, buf_max, l, 0);
 
-  while ((fgets (line, STR_MAX, infile)) != NULL)
+  while ((fgets (line, STR_MAXLONG, infile)) != NULL)
     {
       /* Quote all correspondence so PR doesn't get messed up */
       APPEND_STRING ((char*)" ", buf, b, buf_len, buf_max, l, 0);
@@ -790,6 +948,29 @@ append_report (infile, filename)
   /* Now write out the PR, with that new field.  */
   create_report (filename, 1);
 
+  number = pr_number_from_path (filename);
+
+  /* Now update the index with the new Last-Modified date.  */
+  current_index = get_index ();
+  for (i = current_index; i ; i = i->next)
+    {
+      /* Let's see if we can find what we are looking for.  */
+      if (strcmp (i->number, number) == 0)
+        {
+          t = get_date (date, NULL);         
+          i->last_modified = (char *) xmalloc (18);
+          sprintf (i->last_modified, "%d", t);    
+          /* write out the new index.  */
+          write_index (current_index);
+          break; 
+        }
+    }
+  free_index (current_index);
+  if (i == NULL)
+    {
+      punt (1, "%s: can't find index entry for %s\n", program_name, number);
+    }
+
   /* If it's "foo (Foo Bar)", chop off the rest.  */
   p = field_value (RESPONSIBLE);
   q = (char *) strchr (p, ' ');
@@ -808,7 +989,7 @@ append_report (infile, filename)
 
   if (find_submitter (&submitter, field_value (SUBMITTER)) == -1)
     punt (0, "%s: PR %s has a Submitter field of `%s', which is invalid.",
-	  program_name, field_value (NUMBER), field_value (SUBMITTER));
+	  program_name, number, field_value (SUBMITTER));
   else
     {
       char *addr;
@@ -834,7 +1015,7 @@ append_report (infile, filename)
   fprintf (msg, "\n");
 
   fprintf (msg, "The following reply was made to PR %s/%s; it has been noted by GNATS.\n",
-	   field_value (CATEGORY), field_value (NUMBER));
+	   field_value (CATEGORY), number);
 
   if (fwrite (buf, sizeof (char), buf_len, msg) != buf_len)
     {
@@ -893,3 +1074,4 @@ get_bug_number ()
 
   return bug_number;
 }
+
