@@ -1,5 +1,5 @@
 /* Remote GNATS server.
-   Copyright (C) 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1994, 95, 96, 1997 Free Software Foundation, Inc.
    Contributed by Brendan Kehoe (brendan@cygnus.com).
 
 This file is part of GNU GNATS.
@@ -22,6 +22,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 #include "gnats.h"
 #include "gnatsd.h"
 #include "query.h"
+#include "db_conf.h"
 
 char myname[MAXHOSTNAMELEN];
 
@@ -30,16 +31,28 @@ char *program_name;
 /* If 1, we're running the daemon.  */
 int is_daemon = 1;
 
-/* The name of the host talking with us.  */
+/* If 1, we're running a network client.  */
+int is_client = 0;
+
+/* The name and IP address of the host talking with us and their access level.  */
 char *current_host;
+char *current_addr;
+int   host_access;
+int   user_access;
 
 /* Whether or not we've made sure they are who they say they are.  */
 int host_verified = 0;
 
+/* Whether or not they need to execute CHDB; reset to 0 when they do */
+int require_db    = 0;
+
 struct option long_options[] =
 {
   {"directory", 1, NULL, 'd'},
   {"not-inetd", 0, NULL, 'n'},
+  {"require-db", 0, NULL, 'r'},
+  {"version", 0, NULL, 'V'},
+  {"help", 0, NULL, 'h'},
   {NULL, 0, NULL, 0}
 };
 
@@ -49,7 +62,7 @@ Index *search;
 /* Whether we should emit a linefeed and a newline.  */
 extern int doret;
 
-void usage ();
+void usage (), version ();
 
 #ifdef HAVE_KERBEROS
 /* Non-zero if the client has used Kerberos authentication.  */
@@ -59,9 +72,6 @@ int client_authorized;
 /* Non-zero if the host has to use Kerberos.  */
 int kerberos_host;
 
-#define FLAG_NEED_AUTHORIZATION		1
-#define FLAG_ONE_ARG			2
-
 typedef struct _command
 {
   char *s;
@@ -71,74 +81,99 @@ typedef struct _command
 
 static Command cmds[] =
 {
+  /* Various unrestricted commands */
   { "HELO", GNATS_helo, 0 },
   { "QUIT", GNATS_quit, 0 },
-
-  /* Query a PR.  */
-  { "QURY", GNATS_qury, FLAG_NEED_AUTHORIZATION },
-  { "SUMM", GNATS_summ, FLAG_NEED_AUTHORIZATION },
-  { "FULL", GNATS_full, FLAG_NEED_AUTHORIZATION },
-  { "SQLF", GNATS_sqlf, FLAG_NEED_AUTHORIZATION },
-
-  /* Edit a PR.  */
-  { "LOCK", GNATS_lock, FLAG_NEED_AUTHORIZATION },
-  { "EDIT", GNATS_edit, FLAG_NEED_AUTHORIZATION },
-  { "UNLK", GNATS_unlk, FLAG_NEED_AUTHORIZATION },
-
-  /* Return the people that should be mailed about a PR.  */
-  { "MLPR", GNATS_mlpr, FLAG_NEED_AUTHORIZATION },
-  /* Return the people that should be mailed about a given category.  */
-  { "MLCT", GNATS_mlct, FLAG_NEED_AUTHORIZATION },
-  /* Return the people that should be mailed about a given submitter.  */
-  { "MLSU", GNATS_mlsu, FLAG_NEED_AUTHORIZATION },
-
-  /* Return the type of submitter of a given name.  */
-  { "TYPE", GNATS_type, FLAG_NEED_AUTHORIZATION },
-
-  { "RESP", GNATS_resp, FLAG_NEED_AUTHORIZATION },
-  { "CATG", GNATS_catg, FLAG_NEED_AUTHORIZATION },
-  { "CONF", GNATS_conf, 0 },
-  { "SVTY", GNATS_svty, 0 },
-  { "STAT", GNATS_stat, 0 },
-  { "ORIG", GNATS_orig, FLAG_NEED_AUTHORIZATION|FLAG_ONE_ARG },
-  { "CLSS", GNATS_clss, FLAG_ONE_ARG },
-  { "PRIO", GNATS_prio, 0 },
-  { "SUBM", GNATS_subm, FLAG_NEED_AUTHORIZATION|FLAG_ONE_ARG },
-  { "TEXT", GNATS_text, FLAG_ONE_ARG },
-  { "MTXT", GNATS_mtxt, FLAG_ONE_ARG },
-
-  { "BFOR", GNATS_bfor, FLAG_ONE_ARG },
-  { "AFTR", GNATS_aftr, FLAG_ONE_ARG },
+  { "AUTH", GNATS_auth, 0 },
+  { "USER", GNATS_user, 0 },
+  { "CHEK", GNATS_chek, 0 },
+  { "CHDB", GNATS_chdb, 0 },
+  { "DBLA", GNATS_dbla, 0 },
+  { "HELP", GNATS_help, 0 },
   { "VDAT", GNATS_vdat, FLAG_ONE_ARG },
 
-  { "LCAT", GNATS_lcat, FLAG_NEED_AUTHORIZATION },
-  { "LRES", GNATS_lres, FLAG_NEED_AUTHORIZATION },
-  { "LSUB", GNATS_lsub, FLAG_NEED_AUTHORIZATION },
-
-  /* Stop and start the database.  */
-  { "LKDB", GNATS_lkdb, FLAG_NEED_AUTHORIZATION },
-  { "UNDB", GNATS_undb, FLAG_NEED_AUTHORIZATION },
-
-  { "AUTH", GNATS_auth, 0 },
+  /* Set query constraints (no need to restrict) */
   { "RSET", GNATS_rset, 0 },
   { "NOCL", GNATS_nocl, 0 },
-  { "HELP", GNATS_help, 0 },
-
+  { "RESP", GNATS_resp, 0 },
+  { "CATG", GNATS_catg, 0 },
+  { "SYNP", GNATS_synp, FLAG_ONE_ARG },
+  { "STAT", GNATS_stat, 0 },
+  { "CONF", GNATS_conf, 0 },
+  { "SVTY", GNATS_svty, 0 },
+  { "ORIG", GNATS_orig, FLAG_ONE_ARG },
+  { "RLSE", GNATS_rlse, FLAG_ONE_ARG },
+  { "CLSS", GNATS_clss, FLAG_ONE_ARG },
+  { "PRIO", GNATS_prio, 0 },
+  { "SUBM", GNATS_subm, FLAG_ONE_ARG },
+  { "TEXT", GNATS_text, FLAG_ONE_ARG },
+  { "MTXT", GNATS_mtxt, FLAG_ONE_ARG },
+  { "ABFR", GNATS_abfr, FLAG_ONE_ARG },
+  { "ARAF", GNATS_araf, FLAG_ONE_ARG },
+  { "CBFR", GNATS_cbfr, FLAG_ONE_ARG },
+  { "CAFT", GNATS_caft, FLAG_ONE_ARG },
+  { "MBFR", GNATS_mbfr, FLAG_ONE_ARG },
+  { "MAFT", GNATS_maft, FLAG_ONE_ARG },
 #ifdef GNATS_RELEASE_BASED
+  { "BFOR", GNATS_bfor, FLAG_ONE_ARG },
+  { "AFTR", GNATS_aftr, FLAG_ONE_ARG },
   { "QRTR", GNATS_qrtr, FLAG_ONE_ARG },
   { "KYWD", GNATS_kywd, FLAG_ONE_ARG },
 #endif
 
+  /* Query a PR. */
+  { "QURY", GNATS_qury, ACCESS_VIEW },
+  { "SUMM", GNATS_summ, ACCESS_VIEW },
+  { "FULL", GNATS_full, ACCESS_VIEW },
+  { "SQLF", GNATS_sqlf, ACCESS_VIEW },
+  { "SQL2", GNATS_sql2, ACCESS_VIEW },
+
+  /* List GNATS internals */
+  { "LCAT", GNATS_lcat, ACCESS_VIEW },
+  { "LCFG", GNATS_lcfg, ACCESS_VIEW },
+  { "LCLA", GNATS_lcla, ACCESS_VIEW },
+  { "LRES", GNATS_lres, ACCESS_VIEW },
+  { "LSUB", GNATS_lsub, ACCESS_VIEW },
+  { "LSTA", GNATS_lsta, ACCESS_VIEW },
+
+  /* Return the people that should be mailed about a given PR, category or submitter. */
+  { "MLPR", GNATS_mlpr, ACCESS_VIEW },
+  { "MLCT", GNATS_mlct, ACCESS_VIEW },
+  { "MLSU", GNATS_mlsu, ACCESS_VIEW },
+
+  /* Return the mail address of a responsible person */
+  { "MLRS", GNATS_mlrs, ACCESS_VIEW },
+
+  /* Return the type of submitter of a given name.  */
+  { "TYPE", GNATS_type, ACCESS_VIEW },
+
+  /* Edit a PR. */
+  { "LOCK", GNATS_lock, ACCESS_EDIT },
+  { "EDIT", GNATS_edit, ACCESS_EDIT },
+  { "UNLK", GNATS_unlk, ACCESS_EDIT },
+
+  /* Stop and start the database.  */
+  { "LKDB", GNATS_lkdb, ACCESS_EDIT },
+  { "UNDB", GNATS_undb, ACCESS_EDIT },
+
+  /* List absolute database pathnames */
+  { "DBLS", GNATS_dbls, ACCESS_ADMIN },
+
   { NULL, NULL },
 };
 
 int
-match (line, p)
+match (line, p, matchcase)
      char *line;
      char *p;
+     int   matchcase;
 {
   int matched;
 
+  /* If p is a null string, this is a match (downward compatibility) */
+  if (!*p)
+    return 1;
+  
   for ( ; *p; line++, p++)
     {
       if (*line == '\0' && *p != '*')
@@ -149,8 +184,16 @@ match (line, p)
 	  /* Literal match with following character. */
 	  p++;
 	default:
-	  if (tolower (*line) != tolower (*p))
-	    return 0;
+	  if (matchcase)
+	    {
+	      if (*line != *p)
+	        return 0;
+	    }
+	  else
+	    {
+	      if (tolower (*line) != tolower (*p))
+	        return 0;
+	    }
 	  continue;
 	case '?':
 	  /* Match anything. */
@@ -163,13 +206,17 @@ match (line, p)
 	    /* Trailing star matches everything. */
 	    return 1;
 	  while (*line)
-	    if ((matched = match (line++, p)) != 0)
+	    if ((matched = match (line++, p, matchcase)) != 0)
 	      return matched;
 	  return 0;
 	}
     }
 
-  return 1;
+  /* We've run out of chars in p. If there are more line chars, there is no match */
+  if (*line)
+    return 0;
+  else
+    return 1;
 }
 
 char *
@@ -207,10 +254,71 @@ get_name (host)
   return buf;
 }
 
+void
+set_confidential_access ()
+{
+     if (search->confidential)
+       {
+         xfree (search->confidential);
+	 search->confidential = (char *) NULL;
+       }
+     if (user_access < ACCESS_VIEWCONF)
+       {
+         searching = 1;
+	 search->confidential = (char *) strdup ("no");
+       }
+}
+
+int
+access_level (key)
+     char *key;
+{
+     int access;
+     if (!strcasecmp (key, "none"))
+       access = ACCESS_NONE;
+     else if (!strcasecmp (key, "view"))
+       access = ACCESS_VIEW;
+     else if (!strcasecmp (key, "viewconf"))
+       access = ACCESS_VIEWCONF;
+     else if (!strcasecmp (key, "edit") || !strcasecmp (key, ""))
+       access = ACCESS_EDIT;
+     else if (!strcasecmp (key, "deny"))
+       access = ACCESS_DENY;
+     else if (!strcasecmp (key, "admin"))
+       access = ACCESS_ADMIN;
+     else
+       access = ACCESS_NONE;
+     return (access);
+}
+
+char *
+access_level_str (access)
+     int access;
+{
+     switch (access) {
+     case ACCESS_NONE:     return ("none");
+          break;
+     case ACCESS_VIEW:     return ("view");
+          break;
+     case ACCESS_VIEWCONF: return ("viewconf");
+          break;
+     case ACCESS_EDIT:     return ("edit");
+          break;
+     case ACCESS_DENY:     return ("deny");
+          break;
+     case ACCESS_ADMIN:    return ("admin");
+          break;
+     default: return ("none");
+     }
+}
+
 /* See if we know about this host.  If we don't, then run with it.  */
 int
-allowed (host)
+allowed (host, ipaddr, database, access)
      char *host;
+     char *ipaddr;
+     char *database;
+     int  *access;
 {
   FILE *acc;
   char buffer[BUFSIZ], *x;
@@ -220,12 +328,13 @@ allowed (host)
   char *filename = (char *) alloca (PATH_MAX);
 
   /* The format of the file is:
-        sitename:all:
-     where `sitename' is the hostname allowed; `all' is just a fill-in
-     for now, but may be the field to control what PRs, categories, or
-     submitter-id PRs are allowed; and the final field is blank, but
-     may hold something in the future.  */
-  sprintf (filename, "%s/gnats-adm/%s", gnats_root, ACCESS_FILE);
+        sitename:access:
+     where `sitename' is the hostname allowed; `access' is the access
+     level, and the third field is undefined for now, but may be the
+     field to control what PRs, categories, or submitter-id PRs are
+     allowed, or whatever. */
+  
+  sprintf (filename, "%s/gnats-adm/%s", database, ACCESS_FILE);
   acc = fopen (filename, "r");
   if (acc == (FILE *)NULL)
     {
@@ -254,29 +363,135 @@ allowed (host)
       if (i != 2)
 	continue;
 
-      /* XXX check hostname and IP */
-      if (! match (host, fields[0]))
+      /* check hostname and IP */
+      if (! (match (host, fields[0], 0) || (ipaddr && match (ipaddr, fields[0], 0))))
 	continue;
       found = 1;
+      *access = access_level (fields[1]);
+      if (*access == ACCESS_DENY)
+           found = 0;
+      break;
     }
 
   fclose (acc);
 
 #ifdef HAVE_KERBEROS
-  if (! found)
+  if (! found) {
+    *access = ACCESS_EDIT;
     found = kerberos_host = 1;
+  }
 #endif
 
   return found;
 }
 
+/* Get the access lever for this user. */
+int
+get_user_access (database, filename, user, passwd)
+     char *database;
+     char *filename;
+     char *user;
+     char *passwd;
+{
+  FILE *acc;
+  char buffer[BUFSIZ], *x;
+  char *fields[4], *db;
+  int i, access;
+  int found = 0;
+
+  /* The format of the file is:
+        userid:passwd:access:database */
+
+  acc = fopen (filename, "r");
+  if (acc == (FILE *)NULL)
+    {
+      return (-1);
+    }
+
+  access = -1;
+  
+  while (fgets (buffer, sizeof (buffer), acc) != NULL)
+    {
+      x = strchr (buffer, '\n');
+      if (x != NULL)
+	*x = '\0';
+      x = strchr (buffer, '#');
+      if (x != NULL)
+	*x = '\0';
+      if (buffer[0] == '\0')
+	continue;
+
+      for (fields[0] = buffer, i = 0, x = buffer; *x && i < 3; x++)
+	if (*x == ':')
+	  {
+	    *x = '\0';
+	    fields[++i] = x + 1;
+	  }
+      
+      /* Is there a mistake on the setup of this line? (must have 3 or 4 fields) */
+      if (i < 2 || i > 3)
+	continue;
+
+      /* check userid */
+      if (!match (user, fields[0], 1))
+	continue;
+
+      /* check passwd */
+      if (!match (passwd, fields[1], 1))
+        {
+          access = ACCESS_NONE;
+          break;
+        }
+
+      /* check for optional database alias, but just in the global access file */
+      if (i > 2 && !strcmp (filename, GLOBAL_ACCESS_FILE))
+        {
+	  char alias[STR_MAX];
+	  char *l, *l2;
+          char **aliases = NULL;
+          int  count = 0;
+
+          get_conf_aliases(&aliases, &count);
+          
+          /* Compare all given aliases against all the aliases for the database in use */
+          for (l2 = fields[3], l = get_next_field(l2, alias, ',');
+               l || l2;
+               (l2 = l) && (l = get_next_field(l, alias, ',')))
+            {
+              for (i = 0; i < count; i++)
+                {
+                  if (!strcmp (database, lookup_conf_alias(aliases[i])) &&
+                      match (aliases[i], alias, 1))
+                    {
+                      found = 1;
+                      break;
+                    }
+                }
+              if (found)
+                break;
+            }
+          for (i = 0; i < count; i++)
+            xfree (aliases[i]);
+          xfree((char*) aliases);
+          if (!found)
+            continue;
+        }
+      
+      access = access_level (fields[2]);
+
+      break;
+    }
+
+  fclose (acc);
+
+  return access;
+}
+
 void
 start_connection ()
 {
-  char *host;
   struct sockaddr_in s;
-  int len;
-
+  int len, access;
   len = sizeof (s);
   if (getpeername (0, (struct sockaddr *)&s, &len) < 0)
     {
@@ -287,7 +502,8 @@ start_connection ()
 		  CODE_NO_ACCESS);
 	  exit (1);
 	}
-      host = "stdin";
+      current_host = "stdin";
+      current_addr = current_host;
     }
   else
     {
@@ -299,21 +515,23 @@ start_connection ()
 		  CODE_NO_ACCESS);
 	  exit (1);
 	}
-
-      host = get_name (&s.sin_addr);
-      if (! host)
-	host = (char *) inet_ntoa (s.sin_addr);
+      current_addr = (char *) inet_ntoa (s.sin_addr);
+      current_host = get_name (&s.sin_addr);
+      if (! current_host) {
+        current_host = current_addr;
+      }
     }
 
-  if (!allowed (host))
+  if (!allowed (current_host, current_addr, gnats_root, &access))
     {
-      syslog (LOG_NOTICE, "%s not allowed access", host);
-      printf ("%d You are not on the access list.\r\n", CODE_NO_ACCESS);
+      syslog (LOG_ERR, "host not allowed access: %s (%s)", current_host, current_addr);
+      printf ("%d You are not on the host access list: %s (%s).\r\n",
+              CODE_NO_ACCESS, current_host, current_addr);
       exit (1);
     }
+  host_access = user_access = access;
 
-  current_host = host;
-  syslog (LOG_INFO, "%s connect", host);
+  syslog (LOG_INFO, "host connected: %s (%s)", current_host, current_addr);
 }
 
 
@@ -417,13 +635,13 @@ Argify(line, argvp, cp)
     }
     if (! (*cp)->s)
       *cp = NULL;
-
+  
     if (! p) {
       argv[1] = NULL;
       return 1;
     }
 
-    if (*cp && (*cp)->flags & FLAG_ONE_ARG) {
+    if (*cp && (((*cp)->flags & FLAG_MASK) == FLAG_ONE_ARG)) {
       while (*p && (*p == ' ' || *p == '\n'))
 	p++;
       argv[1] = p;
@@ -458,14 +676,15 @@ main (argc, argv)
   Command *cp;
   MsgType r;
   int optc;
-  int not_inetd = 0;
+  int not_inetd  = 0;
+  int access, access_required;
 
   program_name = basename (argv[0]);
 
   search = (Index *) xmalloc (sizeof (Index));
   memset (search, 0, sizeof (Index));
 
-  while ((optc = getopt_long (argc, argv, "d:n",
+  while ((optc = getopt_long (argc, argv, "d:nrVh",
 			      long_options, (int *) 0)) != EOF)
     {
       switch (optc)
@@ -473,16 +692,28 @@ main (argc, argv)
 	case 'd':
 	  gnats_root = optarg;
 	  break;
+
 	case 'n':
 	  not_inetd = 1;
 	  break;
+
+	case 'r':
+	  require_db = 1;
+	  break;
+
+	case 'V':
+	  version ();
+	  exit (0);
+	  break;
+
+	case 'h':
 	default:
 	  usage ();
 	}
     }
 
-  configure ();
-  init_gnats ();
+  init_gnats (program_name);
+
   umask (022);
   re_set_syntax ((RE_SYNTAX_POSIX_EXTENDED | RE_BK_PLUS_QM) & ~RE_DOT_NEWLINE);
 
@@ -534,15 +765,29 @@ main (argc, argv)
   if (! not_inetd)
     start_connection ();
   else
-    current_host = myname;
-
-  index_chain = get_index ();
-  if (index_chain == NULL)
     {
-      printf ("%d GNATS server cannot read the index.\r\n", CODE_FILE_ERROR);
-      exit (1);
+      /* added host checking to plug a security hole. Otherwise you
+         could fire the daemon up in non inet mode and access any data
+         base. */
+      current_host = myname;
+      if (hp)
+        current_addr = (char *) inet_ntoa (*(struct in_addr *) hp->h_addr);
+      else
+        current_addr = myname;
+      if (!allowed (current_host, current_addr, gnats_root, &access))
+	{
+          printf ("%d You are not on the host access list: %s (%s).\r\n",
+                  CODE_NO_ACCESS, current_host, current_addr);
+	  exit (1);
+	}
+      host_access = user_access = access;
     }
 
+  set_confidential_access ();
+
+/* printf ("host: %s addr: %s access: %d\r\n", current_host, current_addr, host_access); */
+/* fflush(stdout); */
+  
   printf ("%d%c%s GNATS server %s ready.\r\n", CODE_GREETING,
 	  kerberos_host ? '-' : ' ', myname, version_string);
 
@@ -586,26 +831,56 @@ main (argc, argv)
 	  printf ("%d Unrecognized command.\r\n", CODE_ERROR);
 	  continue;
 	}
-      /* Check usage */
-
-      /* Did they say hi? */
+      
+      /* Check access level */
+      access_required = cp->flags & ACCESS_MASK;
+      
+/* printf ("cmd=%s access_required=%d user_access=%d host_access=%d\r\n", */
+/*          av[0], access_required, user_access, host_access); */
+      
 #ifdef HAVE_KERBEROS
-      if (kerberos_host && (cp->flags & FLAG_NEED_AUTHORIZATION)
-	  && !client_authorized)
+      /* Did they send AUTH? */
+      if (kerberos_host && (access_required > user_access) && !client_authorized)
 	{
 	  syslog (LOG_ERR, "GNATS command `%s' without authentication for %s",
 		  cp->s, current_host);
-	  printf ("%d You are not authorized to perform this operation (%s).\r\n",
-		  CODE_NO_ACCESS, cp->s);
-	  continue;
 	}
-      else
 #endif
-	/* Call the function.  We skip the command itself in the arg list.  */
-	(*cp->fn) (ac - 1, av + 1);
+      
+      /* require_db means the first command MUST be CHDB and it must succeed */
+      if (require_db || !strcasecmp (av[0], "chdb")) {
+        if (!strcasecmp (av[0], "chdb"))
+          (*cp->fn) (ac - 1, av + 1);
+        if (require_db)
+          exit (1);
+        continue;
+      }
+      
+      if (access_required > user_access)
+        {
+          printf ("%d You are not authorized to perform this operation (%s).\r\n",
+                  CODE_NO_ACCESS, cp->s);
+          continue;
+        }
+
+      /* For performance, we've delayed reading the index until now in case the user
+         gives a CHDB command first, which is handled above */
+      if (!index_chain) {
+        index_chain = get_index ();
+        /*   if (index_chain == NULL) */
+        /*     { */
+        /*       printf ("%d GNATS server cannot read the index.\r\n", CODE_FILE_ERROR); */
+        /*       exit (1); */
+        /*     } */
+      }
+      
+      /* Call the function.  We skip the command itself in the arg list.  */
+      (*cp->fn) (ac - 1, av + 1);
     }
 
   printf ("%d Later.\r\n", CODE_CLOSING);
+
+  free_db_conf();
   exit (0);
 }
 
@@ -613,7 +888,7 @@ void
 usage ()
 {
   fprintf (stderr, "\
-Usage: %s [-n] [-d directory] [--directory=directory] [--not-inetd]\n",
+Usage: %s [-h] [-n] [-d directory] [--help] [--directory=directory] [--not-inetd]\n",
 	   program_name);
   exit (1);
 }
