@@ -39,6 +39,9 @@ int force = 0;
 /* If 1, we're running the daemon.  */
 int is_daemon = 0;
 
+/* If 1, we're running a network client.  */
+int is_client = 1;
+
 /* The name and port number of the GNATS server.  */
 char *host_name = 0;
 int port = -1;
@@ -49,6 +52,10 @@ int debug = 0;
 /* The descriptor for talking to the server in non-stdio ways.  */
 extern int sockfd;
 
+/* If TRUE, PR is being written to the network and multitext lines beginning
+   with a dot need to be escaped with another dot */
+extern int doescape;
+
 enum {
   NONE,
   LOCK,
@@ -73,6 +80,8 @@ struct option long_options[] =
   {"outfile", 1, NULL, 'o'},
   {"host", 1, NULL, 'H'},
   {"port", 1, NULL, 'P'},
+  {"user", 1, NULL, 'v'},
+  {"passwd", 1, NULL, 'w'},
   {"debug", 0, NULL, 'D'},
   {"version", 0, NULL, 'V'},
   {"help", 0, NULL, 'h'},
@@ -107,7 +116,7 @@ receive_list (cmd, suff)
 static int n_to_validate = 0;
 static void
 do_field (val, command)
-     unsigned char *val, *command;
+     char *val, *command;
 {
   if (! val)
     val = " ";
@@ -130,7 +139,43 @@ net_check_pr (fp)
 {
   char *p;
 
-  init_gnats ();
+  init_pr ();
+  init_header ();
+  init_network ();
+
+  /* Read the message header in.  */
+  if (read_header (fp) < 0)
+    {
+      fprintf (stderr, "%s: end-of-file reading PR\n", program_name);
+      return 0;
+    }
+  read_pr (fp, 0);
+
+  if (debug)
+    fprintf (stderr, "%s: writing `CHEK %s'\n", program_name, p);
+  fprintf (serv_write, "CHEK\r\n");
+  get_reply (); /* XXX */
+  doescape = 1;
+  write_header (serv_write, NUM_HEADER_ITEMS);
+  fprintf (serv_write, "\r\n");
+  write_pr (serv_write, NUM_PR_ITEMS);
+  fprintf (serv_write, ".\r\n");
+  doescape = 0;
+  /* if get_reply finds errors it writes messages and doesn't return */
+  get_reply ();
+  
+  return 1;
+}
+
+int
+XXXnet_check_pr (fp)
+     FILE *fp;
+{
+  char *p;
+
+  init_pr ();
+  init_header ();
+  init_network ();
 
   /* Read the message header in.  */
   if (read_header (fp) < 0)
@@ -154,6 +199,14 @@ net_check_pr (fp)
   if (pr[DATE_REQUIRED].value && pr[DATE_REQUIRED].value[0])
     do_field (pr[DATE_REQUIRED].value, "VDAT");
 #endif
+
+  if (pr[ARRIVAL_DATE].value && pr[ARRIVAL_DATE].value[0])
+    do_field (pr[ARRIVAL_DATE].value, "VDAT");
+  if (pr[LAST_MODIFIED].value && pr[LAST_MODIFIED].value[0])
+    do_field (pr[LAST_MODIFIED].value, "VDAT");
+  if (pr[CLOSED_DATE].value && pr[CLOSED_DATE].value[0])
+    do_field (pr[CLOSED_DATE].value, "VDAT");
+  
   finish_field_validations ();
 
 #if 0
@@ -213,10 +266,12 @@ net_modify_pr (fp)
     fprintf (stderr, "%s: writing `EDIT %s'\n", program_name, p);
   fprintf (serv_write, "EDIT %s\r\n", p);
   get_reply (); /* XXX */
+  doescape = 1;
   write_header (serv_write, NUM_HEADER_ITEMS);
   fprintf (serv_write, "\r\n");
   write_pr (serv_write, NUM_PR_ITEMS);
   fprintf (serv_write, ".\r\n");
+  doescape = 0;
   get_reply ();
 
   return 1;
@@ -234,12 +289,13 @@ main (argc, argv)
   char *username;
   int result = 1, get_lists = 0;
   char *processid = (char *)NULL;
+  char *user = "anonymous", *passwd = "guest";
 
   program_name = basename (argv[0]);
   host_name = gnats_server;
   edit_options = MODIFY;
 
-  while ((option = getopt_long (argc, argv, "l:uLUcgd:f:o:p:H:DVhP:",
+  while ((option = getopt_long (argc, argv, "l:uLUcgd:f:o:p:H:v:w:DVhP:",
 				long_options, (int *)0)) != EOF)
     {
       switch (option)
@@ -308,6 +364,14 @@ main (argc, argv)
 	  port = atoi (optarg);
 	  break;
 
+        case 'v':
+          user = optarg;
+          break;
+
+        case 'w':
+          passwd = optarg;
+          break;
+
 	case 'D':
 	  debug = 1;
 	  break;
@@ -345,7 +409,7 @@ main (argc, argv)
       exit (1);
     }
 
-  client_init ();
+  client_init_gnats (user, passwd);
 
   switch (edit_options) {
     case LOCK:
@@ -410,9 +474,10 @@ usage ()
 {
   fprintf (stderr, "\
 Usage: %s [-cuLUVFh] [-d directory] [-f filename] [-l username] [-o filename]\n\
-          [--directory=directory] [--filename=filename]\n\
+          [-v userid] [-w password] [--directory=directory] [--filename=filename]\n\
           [--lock=user] [--process=pid] [--unlock] [--lockdb] [--unlockdb]\n\
-	  [--host=server] [--outfile=filename] [--help] [--version]\n\
+	  [--host=server] [--port=port] [--outfile=filename] [--help] [--version]\n\
+          [--user=userid] [--passwd=password]\n\
           [--check] [PR]\n", program_name);
 }
 
